package extensions;

import java.io.IOException;
import java.net.Socket;
import java.net.URLDecoder;
import java.util.regex.Pattern;
import java.util.HashMap;
 
import dareka.common.Logger;
import dareka.extensions.Extension;
import dareka.processor.HttpHeader;
import dareka.processor.HttpRequestHeader;
import dareka.processor.Processor;
import dareka.processor.Resource;
import dareka.processor.StringResource;
import dareka.processor.impl.Cache;
 
public class test implements Extension, Processor  {
	private static final Pattern SUPPORT_PATTERN = Pattern.compile(
			"^http://www\\.nicovideo\\.jp/cache/json_(?:move|rm(tmp)?)");
	private static final Pattern VIDEOID_LIST_PATTERN = Pattern.compile(
			"^[a-z]{2}\\d+(?:low)?(?:,[a-z]{2}\\d+(?:low)?)*$");
	public String getVersionString(){
		return "test rev02";
	}
	public Object queryInterface(Extension.Type type) {
		if (type == Extension.Type.Extension1)
			return (Extension) this;
		if (type == Extension.Type.Processor1)
			return (Processor) this;
		return null;
	}
	public String[] getSupportedMethods() {
		return new String[] { HttpHeader.POST };
	}
	public Pattern getSupportedURLAsPattern() {
		return SUPPORT_PATTERN;
	}
	public String getSupportedURLAsString() {
		return null;
	}
	public Resource onRequest(HttpRequestHeader requestHeader, Socket browser) throws IOException {
		String path = requestHeader.getPath();
		if (path.length() < 12)
			return null;
		String mode = path.substring(12);
		String postString = dareka.Main.getRewriterProcessor().getPostString(requestHeader,browser,"UTF-8");
		HashMap<String,String> list = new HashMap<String,String>();
		String key,val;
		String[] param;
		for(String tmp: postString.split("&")){
			param = tmp.split("=",2);
			key = URLDecoder.decode(param[0],"UTF-8");
			if(param.length==2) val = URLDecoder.decode(param[1],"UTF-8");
			else val = "";
			list.put(key,val);
		}
		String movies = list.get("movies");
		StringResource r = null;
		if(movies != null && VIDEOID_LIST_PATTERN.matcher(movies).matches()) {
			if("move".equals(mode)) {
				String target = list.get("target");
				if(target != null)
					r = multiMove(movies,target);
			}
			if("rmtmp".equals(mode)) {
				r = multiRemoveTmp(movies);
			}
			if("rm".equals(mode)) {
				r = multiRemove(movies);
			}
		}
		if(r != null) {
			r.setResponseHeader("Content-Type", "application/json; charset=UTF-8");
		}
		return r;
	}
	private synchronized StringResource multiMove(String movies, String target) throws IOException {
		String success = "[", failure = "[";
		int successCount = 0, failureCount = 0;
		for(String id : movies.split(",")){
			Cache cache = new Cache(id);
			if (cache.exists()){
				if(cache.moveTo(target)) {
					success += "\""+id+"\",";
					successCount++;
				} else {
					failure += "\""+id+"\",";
					failureCount++;
				}
			} else {
				failure += "\""+id+"\",";
				failureCount++;
			}
		}
		String res = "{\n"+
			"\t\"success\":{\n"+
				"\t\t\"list\":"+success.replaceFirst(",$","")+"],\n"+
				"\t\t\"count\":"+successCount+"\n"+
			"\t},\n"+
			"\t\"failure\":{\n"+
				"\t\t\"list\":"+failure.replaceFirst(",$","")+"],\n"+
				"\t\t\"count\":"+failureCount+"\n"+
			"\t}\n"+
		"}";
		return new StringResource(res);
	}
	private synchronized StringResource multiRemoveTmp(String movies) throws IOException {
		String success = "[", failure = "[";
		int successCount = 0, failureCount = 0;
		for(String id : movies.split(",")){
			Cache cache = new Cache(id);
			if (cache.exists()){
				if(Cache.removeTmp(id)) {
					success += "\""+id+"\",";
					successCount++;
				} else {
					failure += "\""+id+"\",";
					failureCount++;
				}
			} else {
				failure += "\""+id+"\",";
				failureCount++;
			}
		}
		String res = "{\n"+
			"\t\"success\":{\n"+
				"\t\t\"list\":"+success.replaceFirst(",$","")+"],\n"+
				"\t\t\"count\":"+successCount+"\n"+
			"\t},\n"+
			"\t\"failure\":{\n"+
				"\t\t\"list\":"+failure.replaceFirst(",$","")+"],\n"+
				"\t\t\"count\":"+failureCount+"\n"+
			"\t}\n"+
		"}";
		return new StringResource(res);
	}
	private synchronized StringResource multiRemove(String movies) throws IOException {
		String success = "[", failure = "[";
		int successCount = 0, failureCount = 0;
		for(String id : movies.split(",")){
			Cache cache = new Cache(id);
			if (cache.exists()){
				if (!id.endsWith("low")) {
					if (Cache.removeAll(id)) {
						success += "\""+id+"\",";
						successCount++;
					} else {
						failure += "\""+id+"\",";
						failureCount++;
					}
				} else if (Cache.remove(id)) {
					success += "\""+id+"\",";
					successCount++;
				} else {
					failure += "\""+id+"\",";
					failureCount++;
				}
			} else {
				failure += "\""+id+"\",";
				failureCount++;
			}
		}
		String res = "{\n"+
			"\t\"success\":{\n"+
				"\t\t\"list\":"+success.replaceFirst(",$","")+"],\n"+
				"\t\t\"count\":"+successCount+"\n"+
			"\t},\n"+
			"\t\"failure\":{\n"+
				"\t\t\"list\":"+failure.replaceFirst(",$","")+"],\n"+
				"\t\t\"count\":"+failureCount+"\n"+
			"\t}\n"+
		"}";
		return new StringResource(res);
	}
}
